/**
 * Copyright 2016 - present, Matthieu Jabbour <matthieu.jabbour@gmail.com>.
 * All rights reserved.
 */


/**
 * This function checks user config and throws an error if it is not valid, to prevent
 * running misconfigured webpack (which can lead to bad side effects,
 * like unexpected directories deletion).
 * @param {string} userConfig User configuration for webpack.
 * @returns {void}
 */
module.exports = (userConfig) => {
  const allowedTargets = ['web', 'node', 'electron'];

  // Checking project target...
  if (allowedTargets.indexOf(userConfig.target) < 0) {
    throw new Error('User config validation - specified "target" is not a valid target');
  }

  // Checking dev server IP adress and port...
  if (
    userConfig.target === 'web' &&
    typeof userConfig.devServer !== 'object' &&
    typeof userConfig.devServer.ip !== 'string' &&
    typeof userConfig.devServer.port !== 'number'
  ) {
    throw new Error('User config validation - "devServer" params are not valid');
  }

  // Checking source path...
  if (typeof userConfig.srcPath !== 'string') {
    throw new Error('User config validation - "srcPath" is not a valid path');
  }

  // Checking entries...
  if (typeof userConfig.entry !== 'object') {
    throw new Error('User config validation - "entry" is not a valid entries object');
  }

  // Checking dist path...
  if (typeof userConfig.distPath !== 'string') {
    throw new Error('User config validation - "distPath" is not a valid path');
  }

  // Checking banner message...
  if (userConfig.banner !== undefined && typeof userConfig.banner !== 'string') {
    throw new Error('User config validation - "banner" is not a valid banner');
  }
};
