/**
 * Copyright 2016 - present, Matthieu Jabbour <matthieu.jabbour@gmail.com>.
 * All rights reserved.
 */


/* eslint-disable no-console, import/no-unresolved */


process.env.NODE_ENV = 'development';


const path = require('path');
const fs = require('fs-extra');
const webpack = require('webpack');
const express = require('express');
const packageJson = require('../../../package.json');
const webpackDevMiddleware = require('webpack-dev-middleware');
const webpackHotMiddleware = require('webpack-hot-middleware');
const config = require('../config/webpack.config.dev');


const types = 'types.d.ts';
const compiler = webpack(config);
const distPath = config.output.path;
const srcPath = config.context;


// For front-end projects, we setup a dev server to leverage on HMR and Hot Reloading.
if (config.target === 'web') {
  const server = express();

  // Removing existing `dist` directory...
  fs.remove(distPath)
    // Running webpack compiler...
    .then(() => {
      // `webpack-dev-middleware` package enables Webpack to be run as a server in watch mode,
      // for a better development experience.
      // We use it instead of `webpack-dev-server` package because it offers more flexibility and
      // control over the server.
      server.use(webpackDevMiddleware(compiler, {
        stats: {
          colors: true,
          cached: true,
          performance: true,
          modules: false,
          children: false,
        },
        publicPath: '/assets',
        index: 'index.html',
      }));

      // `webpack-hot-middleware` package enables HMR, along with a dev server.
      server.use(webpackHotMiddleware(compiler, {
        path: '/assets/_hmr',
        heartbeat: 2000,
      }));

      server.use(express.static(path.resolve(config.output.path, '..')));
      server.listen(config.devServer.port, config.devServer.ip, () => {
        console.log(`Starting dev server at http://${config.devServer.ip}:${config.devServer.port}...`);
      });
    })
    // If any error occurs...
    .catch((error) => {
      console.error(error.message);
    });
} else {
  // `package.json` content used for distribution.
  const distPackageJson = JSON.stringify({
    main: `./${packageJson.name}.js`,
    types: `./${types}`,
    name: packageJson.name,
    bugs: packageJson.bugs,
    author: packageJson.author,
    version: packageJson.version,
    engines: packageJson.engines,
    licence: packageJson.licence,
    keywords: packageJson.keywords,
    homepage: packageJson.homepage,
    repository: packageJson.repository,
    description: packageJson.description,
    contributors: packageJson.contributors,
    dependencies: packageJson.dependencies,
  });


  // Removing existing `dist` directory...
  fs.remove(distPath)
    // Running webpack compiler...
    .then(() => {
      compiler.watch({}, (error, stats) => {
        console.log(stats.toString({
          colors: true,
          cached: true,
          performance: true,
          modules: false,
          children: false,
        }));

        if (!stats.hasErrors()) {
          // Writing distributable `package.json` and `types.d.ts` files into `dist` directory...
          try {
            fs.writeFileSync(path.join(distPath, 'package.json'), distPackageJson);
            fs.copySync(path.join(srcPath, types), path.join(distPath, types));
          } catch (fsError) {
            console.error(fsError.message);
          }
        }
      });
    })
    // If any error occurs...
    .catch((error) => {
      console.error(error);
    });
}
